/*
   Copyright (C) 2006 by krasnop@bellsouth.net (Alexei Krasnopolski)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License. 
*/

#include "AObject.hpp"
#include "pc.hpp" // for debug only

AObject::AObject( DWORD prio, Hsm* hsm )
  : Thread( prio ),
    list( AO_LISTENERS_LIST_LENGTH ),
    ringBuffer( AO_RINGBUFFER_LENGTH )
{
  stateMachine = hsm;
  typedef void (*fp)( AObject * );           // helper typedef for casting function run()
  Thread::init( this, (fp)(&AObject::run) ); // init thread stack
  stateMachine->setActiveObject( this ); 
}

AObject::AObject( DWORD prio )
  : Thread( prio ),
    list( AO_LISTENERS_LIST_LENGTH ),
    ringBuffer( AO_RINGBUFFER_LENGTH )
{
  stateMachine = (Hsm*)0;
  typedef void (*fp)( AObject * );
  Thread::init( this, (fp)(&AObject::run) );
}

void
AObject::run()
{
  Event ev;
  int failedProcess = 0;
  while( 1 == 1 )     // this is infinite loop
  {
//debugPrint( 55+getPriority(), 'A'+getPriority() );
    if( ringBuffer.read( &ev ) == 0 )  // try to read event from buffer
    {
      ready = 0;          // have no an events to process
      AO_CONTEXT_SW();    // pass CPU control to others AO by invoking of scheduler
    }
    else
    {
      Event rev( ev );                // create a clone of ev;
      if( processEvent( &ev ) == 0 )
      {                               // processEvent() can not comlete proceeding of the event
        receiveEvent( &rev );         // return the event back to the buffer ( to tail)
        failedProcess++;
        if( ringBuffer.bufferLoad() <= failedProcess )
        {      
          failedProcess = 0;
          ready = 0;                    // allow scheduler to activate another AO
          AO_CONTEXT_SW();              // schedule a tasks ...
        }
      }
    }
  }
}

/**
 * If we do not use state machine template ( stateMachine = null ) we have to override this method
 * in superclass 
 */
DWORD 
AObject::processEvent( Event * ev )
{   
  return stateMachine->dispatchEvent( ev ); 
}

void
AObject::publishEvent( Event * ev )
{
  for( int i = 0; i < list.length(); i++ )  // send message to the all AO that listen to this object
    list.elementAt( i )->receiveEvent( ev );
}

void
AObject::addListener( AObject * obj )
{
  list.add( obj );
}

void
AObject::removeListener( AObject * obj )
{
  list.remove( obj );
}

void
AObject::receiveEvent( Event * ev )
{
  if( ringBuffer.write( ev ) == 0 )  // put incoming message to the buffer for further processing
  {
    // buffer is full !!! - ignores the event or/and notifies a source of the event;
  }
  ready = 1; // this AO is ready to run. Scheduler will give it control during next schedule time when this priority will be highest in the system.
}
