/*
   Copyright (C) 2006 by krasnop@bellsouth.net (Alexei Krasnopolski)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License. 
*/

/*
***************************************************************************
*                          DATA TYPES
*                     (Compiler Specific)
***************************************************************************
*/

typedef unsigned char    BOOLEAN;
typedef unsigned char    BYTE;                     /* Unsigned  8 bit type */
typedef signed   char    BYTE_S;                   /* Signed    8 bit type */
typedef unsigned short   WORD;                     /* Unsigned 16 bit type */
typedef signed   short   WORD_S;                   /* Signed   16 bit type */
typedef unsigned long    DWORD;                    /* Unsigned 32 bit type */
typedef signed   long    DWORD_S;                  /* Signed   32 bit type */

typedef unsigned long    AO_STACK;                 /* Each stack entry is 32-bit wide */

/*
***************************************************************************
*                   Intel 80x86 (Protected-Mode, Flat Model)
*  Macroses for port access:
*                    
***************************************************************************
*/
#define inp( _register_, _value_ )   \
{                                               \
    asm volatile ( "xor %%eax,%%eax ;"          \
                   "inb %%dx, %%al"             \
                   : "=a" (_value_)             \
                   :  "d"(_register_)           \
        );                                      \
}

#define outp( _register_, _value_ )          \
{                                                       \
    asm volatile ( "outb %%al,%%dx"                     \
                   :                                    \
                   : "a" (_value_), "d"(_register_) \
        );                                              \
}

/*
***************************************************************************
*                   Intel 80x86 (Protected-Mode, Flat Model)
* 
* CPU interrupt enable/disable macros:
* Disable/Enable interrupts using simple instructions.  
* 
***************************************************************************
*/

/* Disable interrupts                        */
#define  ENTER_CRITICAL()                       \
{                                               \
    asm("cli");                                 \
}
/* Enable  interrupts                        */
#define  EXIT_CRITICAL()                        \
{                                               \
    asm("sti");                                 \
}

/*
*********************************************************************************************************
*                           Intel 80x86 (Protected-Mode, Flat Model)
* Context switching by using program interrupt 0x30 
*********************************************************************************************************
*/

#define  AO_CONTEXT_SW()               \
{                                      \
    asm("int $0x30");                  \
}
