;--------------------------------------------------------------------------
;   Copyright (C) 2006 by krasnop@bellsouth.net (Alexei Krasnopolski)
;
;   Licensed under the Apache License, Version 2.0 (the "License");
;   you may not use this file except in compliance with the License.
;   You may obtain a copy of the License at
;
;       http://www.apache.org/licenses/LICENSE-2.0
;
;   Unless required by applicable law or agreed to in writing, software
;   distributed under the License is distributed on an "AS IS" BASIS,
;   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
;   See the License for the specific language governing permissions and
;   limitations under the License. 
;--------------------------------------------------------------------------

; ------------------ M A C R O S -------------------------------------
; macro for interrupt service routine
%macro  pc_irq 2
pc_irq%1:
               push    eax   
               mov      al, 0x20       ; non-specific EOI
%if ( %2 >= 0 ) && ( %2 < 8 )
               out      0x20, al       ;
%else
               out      0xA0, al       ;
%endif
               pop     eax
               iret
%endmacro
; ------------------ M A C R O  #1 -------------------------------------
; macro for mask of interrupt controller 8259
%macro intc_mask_int 1
%if (%1 >= 0x20) && (%1 < 0x28)
%assign port 0x21
%else
%assign port 0xA1
%endif
               mov     ah, (1 << ( %1 & 0x07 ))
               in      al, port
               or      al, ah
               out     port, al
%endmacro
; ------------------ M A C R O  #2 -------------------------------------
; macro for unmask of interrupt controller 8259
%macro intc_unmask_int 1
%if (%1 >= 0x20) && (%1 < 0x28)
%assign port 0x21
%else
%assign port 0xA1
%endif
               mov     ah, ~(1 << ( %1 & 0x07 ))
               in      al, port
               and     al, ah
               out     port, al
%endmacro
; ------------------ M A C R O  #3 -------------------------------------
; my macro for acknoledge of interrupt controller
%macro intc_ack 1
               mov      al, 0x20       ; non-specific EOI
%if (%1 >= 0x20) && (%1 < 0x28)
               out      0x20, al       ;
%else
               out      0xA0, al       ;
%endif
%endmacro
; ------------------ E N D   O F   M A C R O S -------------------------------------

segment         code
                BITS 16
; ------------------ M E M O R Y   M A P -------------------------------------
stackStart           EQU     0x9FFF0  
gdtStart             EQU     0x800    
idtStart             EQU     0        
A_7C00               EQU     0x7C00   ; This is a value of code segment register (CS) when this module is running in real mode 
Display_Buffer       equ     0xB8000

; ------------------ E L F   O F F S E T S -------------------------------------
; offsets for variables in ELF header :
entry_address        EQU     0x18     ; 4
prog_hdr_off         EQU     0x1C     ; 4
sect_hdr_off         EQU     0x20     ; 4
header_size          EQU     0x28     ; 2 
prog_hdr_entry_size  EQU     0x2A     ; 2
prog_hdr_entry_count EQU     0x2C     ; 2
sect_hdr_entry_size  EQU     0x2E     ; 2
sect_hdr_entry_count EQU     0x30     ; 2
sect_name_entr_index EQU     0x32     ; 2

; offsets for variables in a program segment headers
segment_type         EQU     0x0      ; 4 
offset_in_file       EQU     0x4      ; 4
phys_address         EQU     0xC      ; 4
segm_size_in_file    EQU     0x10     ; 4

; offsets for variables in a section headers
section_name         EQU     0x0      ; 4
section_type         EQU     0x4      ; 4
section_flags        EQU     0x8      ; 4
section_address      EQU     0xC      ; 4
section_file_offset  EQU     0x10     ; 4
section_size         EQU     0x14     ; 4

              ALIGN  4, db 0
gdt:               ; memory image of global descriptor table register
                   dw   gdtEnd - gdtStartIm - 1
                   dd   gdtStart                          ;

              ALIGN  4, db 0
idt:               ; memory image of interrupt descriptor table register
                   dw   0x07FF                            ; space for 256 entries
                   dd   idtStart

idtStartIm:
                   dd pc_exception_noerr         ;VECTOR_DIV0                    0
                   dd pc_exception_noerr         ;VECTOR_DEBUG                   1
                   dd pc_exception_noerr         ;VECTOR_NMI                     2
                   dd pc_exception_noerr         ;VECTOR_BREAKPOINT              3
                   dd pc_exception_noerr         ;VECTOR_OVERFLOW                4
                   dd pc_exception_noerr         ;VECTOR_BOUND                   5
                   dd pc_exception_noerr         ;VECTOR_OPCODE                  6
                   dd pc_exception_noerr         ;VECTOR_NO_DEVICE               7
                   dd pc_exception_err           ;VECTOR_DOUBLE_FAULT            8
                   dd pc_exception_noerr
                   dd pc_exception_err           ;VECTOR_INVALID_TSS            10
                   dd pc_exception_err           ;VECTOR_SEGV                   11
                   dd pc_exception_err           ;VECTOR_STACK_FAULT            12
                   dd pc_exception_err           ;VECTOR_PROTECTION             13
                   dd pc_exception_err           ;VECTOR_PAGE                   14
                   dd pc_exception_noerr
                   dd pc_exception_noerr         ;VECTOR_FPE                    16
                   dd pc_exception_err           ;VECTOR_ALIGNMENT              17
                   dd pc_exception_noerr
                   dd pc_exception_noerr
                   dd pc_exception_noerr
                   dd pc_exception_noerr
                   dd pc_exception_noerr
                   dd pc_exception_noerr
                   dd pc_exception_noerr
                   dd pc_exception_noerr
                   dd pc_exception_noerr
                   dd pc_exception_noerr
                   dd pc_exception_noerr
                   dd pc_exception_noerr
                   dd pc_exception_noerr
                   dd pc_exception_noerr
                   dd timer_vsr                   ;INTERRUPT_TIMER            IRQ0 (20)
                   dd pc_irq1                     ;INTERRUPT_KEYBOARD         IRQ1
                   dd pc_irq2                     ;INTERRUPT_SLAVE8259        IRQ2
                   dd pc_irq3                     ;INTERRUPT_COM2             IRQ3
                   dd pc_irq4                     ;INTERRUPT_COM1             IRQ4
                   dd pc_irq5                     ;INTERRUPT_LPT2             IRQ5
                   dd pc_irq6                     ;INTERRUPT_FDD              IRQ6
                   dd pc_irq7                     ;INTERRUPT_LPT1             IRQ7
                   dd pc_irq8                     ;INTERRUPT_WALLCLOCK        IRQ8
                   dd pc_irq9                     ;INTERRUPT_SLAVE8259REDIR   IRQ9
                   dd pc_irqA                     ;                           IRQ10
                   dd pc_irqB                     ;                           IRQ11
                   dd pc_irqC                     ;                           IRQ12
                   dd pc_irqD                     ;INTERRUPT_COPRO            IRQ13
                   dd pc_irqE                     ;INTERRUPT_HDD              IRQ14
                   dd pc_irqF                     ;                           IRQ15
                   dd pr_int                      ; PROGRAMM INTERRUPT              (30)
idtEnd:

PM_set_start:
;   relocation of GDT and program code in proper place for 32bit flat memory model. 
;   set ES == gdtStart
                   cli
                   mov     eax, gdtStart
                   shr     eax, 4                        ;
                   mov     es, ax                        ;
                   mov     di, 0                         ; destination es:di
                   mov     si, gdtStartIm                ; source      ds:si
                   mov     cx, program_end - gdtStartIm  ; number of bytes
               rep movsb
; fill out the whole IDT (256 entries)
                   mov     eax, idtStart
                   shr     eax, 4                        ;
                   mov     es, ax                        ;
                   mov     di, 0
                   mov     cx, 256
.loop:
                   xor     ax, ax
                   stosw                                 ; offset 15-0
                   mov     ax, 8
                   stosw                                 ; selector
                   mov     ax, 0x8E00
                   stosw                                 ; P=1, DPL=0, interrupt gate
                   xor     ax, ax
                   stosw                                 ; offset 31-16
                   loop    .loop
; set up interrupt gates of the IDT for AO-RTOS
                   mov     di, 0                         ; destination es:di
                   mov     si, idtStartIm                ; source      ds:si
cycle:
                   lodsd
                   add     eax, A_7C00                  ; value of labels in compile time have to be increased in run time: in real mode cs = 7c0, in pm cs = 0
                   stosw                                ; offset 15-0
                   mov     ax, 8
                   stosw                                ; selector
                   mov     ax, 0x8E00
                   stosw                                ; P=1, DPL=0, interrupt gate
                   shr     eax, 16
                   stosw                                ; offset 31-16
                   cmp     si, idtEnd
                   jl      cycle
;   # load GDTR and IDTR
                   cli
                   lgdt    [gdt]
                   lidt    [idt]
;   # Switch to protected mode.
                   mov     eax, cr0                       ;
                   or      al,  1                         ;
                   mov     cr0, eax                       ;
               jmp     dword 0x8:(next + A_7C00)
;    # We arrive here in protected mode
next:
               BITS 32
;    # Load data selectors
               mov     ax, 0x10                     ;
               mov     ds, ax                       ;
               mov     es, ax                       ;
               mov     fs, ax                       ;
               mov     gs, ax                       ;
               mov     ss, ax
;    # Set up SP
               mov     esp, stackStart              ;
;    # Reset the flags register. 
               push    0                            ;
               popf                                 ;

;    #  The interrupt controller is configured so that IRQ levels 0-7 trigger
;    #  interrupt vector 32-39; levels 8-15 trigger 40-47.
               mov     al, 0x11       ; 1 -  ICW4 is needed
                                      ; 0 -  cascading I8259
                                      ; 0 -  8 byte interrupt vector
                                      ; 0 -  edge triggered mode
                                      ; 1 -  must be 1 for ICW1
                                      ; 0 -
                                      ; 0 -
                                      ; 0 -
               out     0x20, al       ; ICW 1
               mov     al, 0x20       ; A7-A3 =  20(32.)
               out     0x21, al       ; ICW 2
               mov     al, 0x04       ; irq 2 (34.) has slave
               out     0x21, al       ; ICW 3
               mov     al, 0x01       ; 1 - 80x86 mode
                                      ; 0 - normal EOI
                                      ; 00 - not buffered
                                      ; 0 - sequential
               out     0x21, al       ; ICW 4
 ;   #  Mask off all interrupts except 2.
               mov     al, 0xFB       ;
               out     0x21, al       ; OCW 1

               mov     al, 0x11       ;
               out     0xA0, al       ; ICW 1
               mov     al, 0x28       ;
               out     0xA1, al       ; ICW 2
               mov     al, 0x02       ;
               out     0xA1, al       ; ICW 3
               mov     al, 0x01       ;
               out     0xA1, al       ; ICW 4
;    #   Mask off all interrupts.
               mov     al, 0xFF       ;
               out     0xA1, al       ; OCW 1
;    #   set clock
PC_PIT_CONTROL    EQU   (0x43)
PC_PIT_CLOCK_0    EQU   (0x40)
PC_PIT_CLOCK_1    EQU   (0x41)
PC_PIT_CLOCK_2    EQU   (0x42)
               mov     al, 0x34
               out     PC_PIT_CONTROL, al
               mov     ax, 11932
               out     PC_PIT_CLOCK_0, al
               shr     ax, 8
               out     PC_PIT_CLOCK_0, al

;    #   unmask int 32
               intc_unmask_int 32
               sti

; go to application
; RELOCATION file.elf from temp buffer to original (physical) address space
; Keep in mind that temp buffer and physical addresses are both absolute addresses in flat memory (32bit PM)
               mov     ebx, [temp_buffer + prog_hdr_off]            ; program segment header pointer (offset value)
               add     ebx, temp_buffer                             ; absolute value
               movzx   ecx, word [temp_buffer + prog_hdr_entry_count] ; number of entries in program segment header
.lookup_in_segm:
               cmp     dword [ebx], 1                           ; is type of program segment "load" ?
               jne     .next                                    ; no - go to .next
; relocating of program segment from temp buffer to executable address space
               push    ecx
               cld
               mov     esi, [ebx + offset_in_file]       ; offset of code (program) segment in file
               add     esi, temp_buffer                  ; absolute value
               mov     edi, [ebx + phys_address]         ; physical address of the code segment in memory
               mov     ecx, [ebx + segm_size_in_file]    ; size of code segment
          rep  movsb
               pop     ecx
.next:
               movzx   eax, word [temp_buffer + prog_hdr_entry_size]  ; size of current entry in program header
               add     ebx, eax                                       ; point to the next entry
               loop    .lookup_in_segm                                ; go to process next entry

sections:
; point ebx to a section header
               mov     ebx, [temp_buffer + sect_hdr_off]
               add     ebx, temp_buffer
; point edx to the header entry of the name section 
               mov     cx,  word [temp_buffer + sect_name_entr_index]	
               movzx   eax, word [temp_buffer + sect_hdr_entry_size]
               imul    ax,  cx
               add     eax, ebx                         ; points to a name section header
; point edx to the name section body 
               mov     edx, [eax + section_file_offset]
               add     edx, temp_buffer                 ; points to a body of name section
               
               movzx   ecx, word [temp_buffer + sect_hdr_entry_count] ; number of entries in section header
.lookup_in_sect:
               cmp     dword [ebx + section_type], 1                  ; is a type of the section "PROGBITS" ?
               jne     .next
; find a section with name = ".init"
               mov     esi, [ebx + section_name]											; index in name section body
               add     esi, edx
               mov     edi, initSect + A_7C00
               push    ecx
               
               mov     ecx, 5
         repe  cmpsb    
               pop     ecx	
               je      .find
.next:                         
               movzx   eax, word [temp_buffer + sect_hdr_entry_size]  ; size of current entry in section header
               add     ebx, eax                                       ; point to the next entry
               loop    .lookup_in_sect                                ; go to process next entry
               jmp     $																							; error: can not find .init section
.find:               
; prepares to call .init (embedds a 'ret' instruction after end of the section)
               mov     edi, [ebx + section_address]
               add     edi, [ebx + section_size]             ; points to the section's end
               mov     eax, [edi]                            ; save a first dword of next section 
               mov     [save + A_7C00], eax
               mov     eax, [retInst + A_7C00]
               mov     [edi], eax
               
               push    edi
               mov     esi, [ebx + section_address]
               call    esi									 ; call .init section ELF (on host it does underlying OS)									; call .init module
               pop     edi
                
               mov     eax,   [save + A_7C00]                 ; restore a first dword of next section 
               mov     [edi], eax
            
;               mov     esi, [temp_buffer + entry_address]
;               mov     cx, 8
;               call    displayHex

               mov     esi, [temp_buffer + entry_address]     ; entry (start point of the application) address in elf header
               jmp     esi                                    ; GO TO AO-RTOS application !!!

initSect:      db      ".init"
save:          dd      0
retInst:       ret  
;-------------------------------------------------------------------------------
               TIMES   0x800 - ($ - $$)   db 0
gdtStartIm:
;    /* Selector 0x00 == invalid. */
                   dw   0x0000
                   dw   0x0000
                   db   0x00
                   db   0x00
                   db   0x00
                   db   0x00
;    /* Selector 0x08 == code. */
                   dw   0xFFFF                      ; limit 15-0
                   dw   0x0000                      ; base  15-0
                   db   0x00                        ; base  23-16
                   db   0x9B                        ; AR: P=1, DPL=0, S=1, type=101(code, ER), A=1
                   db   0xCF                        ; G=1(page granularity), D=1(32bit mode), limit 19-16
                   db   0x00                        ; base 31-24
                                                    ; base=0x00000000; limit=0xFFFFF
;    /* Selector 0x10 == data. */
                   dw   0xFFFF
                   dw   0x0000
                   db   0x00
                   db   0x93                        ; AR: P=1, DPL=0, S=1, type=001(data, WR), A=1
                   db   0xCF
                   db   0x00
                                                    ; base=0x00000000; limit=0xFFFFF
;    /* Selector 0x18 == shorter code: faults any code access 0xF0000000-0xFFFFFFFF. */
                   dw   0xFFFF
                   dw   0x0000
                   db   0x00
                   db   0x9B                        ; AR: P=1, DPL=0, S=1, type=101(code, ER), A=1
                   db   0xC7
                   db   0x00
                                                     ; base=0x00000000; limit=0x7FFFF
;    /* Selector 0x20 == data; faults any access 0xF0000000-0xFFFFFFFF. */
                   dw   0xFFFF
                   dw   0x0000
                   db   0x00
                   db   0x93                        ; AR: P=1, DPL=0, S=1, type=001(data, WR), A=1
                   db   0xC7
                   db   0x00
gdtEnd:                                             ; base=0x00000000; limit=0x7FFFF

symbol:        db      '1'
second:        db      100

timer_vsr:
               push    eax
               push    ebx
                 
               intc_ack 32
               sti
               
               dec     byte [second]
               jne     .exit
               mov     byte [second], 100
               mov     ah, 0x07
               mov     al, [symbol]
               xor     al, 0x01
               mov     [symbol], al
               mov     ebx, Display_Buffer+79*2         ;
               mov     [ebx], ax
.exit:
               pop     ebx
               pop     eax
               iret

;timer_os:
;               push   eax   
;               intc_ack 0x20
;               pop     eax    
;               iret

trap_error_code:
               dd      0
; exception handler (no error code)
pc_exception_noerr:
               mov     dword [trap_error_code], 0
               iret

; exception handler (with error code)
pc_exception_err:
               pop     dword [trap_error_code]
               iret

; irq handler (no error code)
pc_irq 1,1
pc_irq 2,2
pc_irq 3,3
pc_irq 4,4
pc_irq 5,5
pc_irq 6,6
pc_irq 7,7
pc_irq 8,8
pc_irq 9,9
pc_irq {A},10
pc_irq {B},11
pc_irq {C},12
pc_irq {D},13
pc_irq {E},14
pc_irq {F},15
pr_int:        iret
